﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Input;
using Ozeki.MediaGateway;
using System.Security;

namespace SilverlightSIPClient
{
    public partial class MainPage : UserControl, INotifyPropertyChanged
    {

        SIPClient sipClient;
        Microphone microphone;
        AudioPlayer audioPlayer;

        PhoneState phoneState;
        
        public MainPage()
        {
            Loaded += MainPage_Loaded;
            InitializeComponent();
        }

        #region EventHandlers

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            phoneState = PhoneState.Normal;
            numpad.owner = this;
            display.DataContext = this;
            UserName = "unregistered";

            microphone = Microphone.GetMicrophone();
            audioPlayer = new AudioPlayer();

            sipClient = new SIPClient(microphone, audioPlayer);

            CallInfo = "Registering...";

            sipClient.ConnectionStateChanged += sipClient_ConnectionStateChanged;
            sipClient.RegistrationStateChanged += sipClient_RegistrationStateChanged;
            sipClient.PhoneLineStateChanged += sipClient_PhoneLineStateChanged;
            sipClient.CallStateChanged += sipClient_CallStateChanged;
            sipClient.CallErrorOccurred += sipClient_CallErrorOccurred;
            sipClient.SIPUserNameReceived += sipClient_SIPUserNameReceived;
            sipClient.IncomingCall += sipClient_IncomingCall;
			sipClient.DTMFReceived += sipClient_DTMFReceived;
            sipClient.Connect();
			
        }
        /// <summary>
        /// Indicates DTMF received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sipClient_DTMFReceived(object sender, GenericEventArgs<int> e)
        {
            Invoke(() =>
            {
                CallInfo = String.Format("DTMF signal received: {0} ", e.Item);
            });
        }

        /// <summary>
        /// Indicates the registered SIP username.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sipClient_SIPUserNameReceived(object sender, GenericEventArgs<string> e)
        {
            Invoke(() =>
            {
                UserName = e.Item.ToString();
            });
        }

        /// <summary>
        /// Indicates call errors. For example : The called number can not be found, equal with Callstate.NotFound
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sipClient_CallErrorOccurred(object sender, GenericEventArgs<CallError> e)
        {
            phoneState = PhoneState.Normal;
            Invoke(() =>
            {
                CallInfo = e.Item.ToString();
            });
        }

        /// <summary>
        /// Indicates for the client, his call state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sipClient_CallStateChanged(object sender, GenericEventArgs<CallState> e)
        {
            if ((int)e.Item < (int)CallState.InCall)
                phoneState = PhoneState.Normal;
            if (e.Item == CallState.InCall)
                phoneState = PhoneState.InCall;

            Invoke(() =>
            {
                CallInfo = e.Item.ToString();
            });
        }

        /// <summary>
        /// Indicates for the client, his registration state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sipClient_RegistrationStateChanged(object sender, GenericEventArgs<RegistrationState> e)
        {
            Invoke(() =>
            {
                switch (e.Item)
                {
                    case RegistrationState.ConnectionLimit:
                        MessageBox.Show("Connection limit");
                        break;
                }
            });

        }
        /// <summary>
        /// Indicates phoneline information changes.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sipClient_PhoneLineStateChanged(object sender, GenericEventArgs<PhoneLineState> e)
        {
            Invoke(() =>
            {
                switch (e.Item)
                {
                    case PhoneLineState.NoRegNeeded:
                    case PhoneLineState.RegistrationSucceded:
                        CallInfo = "Registration success";
                        break;
                    case PhoneLineState.RegistrationFailed:
                        CallInfo = "Registration failed";
                        break;
                    default:
                      CallInfo = e.Item.ToString();
                        break;
                }
            });

        }

        void sipClient_ConnectionStateChanged(object sender, GenericEventArgs<ConnectionState> e)
        {
            Invoke(() =>
            {
                switch (e.Item)
                {
                    case ConnectionState.Success:
                        CallInfo = "Connection success";
                        ConnectionStatus = "Online";
                        break;
                    case ConnectionState.Failed:
                        ConnectionStatus = "Offline";
                        CallInfo = "Connection failed";
                        break;
                    case ConnectionState.Closed:
                        ConnectionStatus = "Offline";
                        CallInfo = "Connection closed";
                        break;
                }
            });
        }
        /// <summary>
        /// Inditcate the incoming call request.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void sipClient_IncomingCall(object sender, GenericEventArgs<string> e)
        {
            Invoke(()=>
                       {
                           CallInfo = "Incoming call from " + e.Item;
                           phoneState = PhoneState.IncomingCall;
                       });


        }

#endregion

        void Invoke(Action action)
        {
            if (Dispatcher.CheckAccess())
                action();
            else
                Dispatcher.BeginInvoke(action);
        }

#region  methods

        /// <summary>
        /// Makea an outgoing call with Callnumber or accepts an incoming call requst.
        /// </summary>
        public void PickUp()
        {
            if (!Microphone.GetPermissionToMicrophone())
            {
                CallInfo = "Check your microphone";
                return;
            }

            if (phoneState == PhoneState.Normal)
            {
                if (string.IsNullOrWhiteSpace(CallNumber))
                    return;

                phoneState = PhoneState.InCall;
                sipClient.Call(CallNumber);
            }
            else if (phoneState == PhoneState.IncomingCall)
            {
                phoneState = PhoneState.InCall;
                sipClient.Accept();
            }
        }
        /// <summary>
        /// Hangup the call or reject incoming call request.
        /// </summary>
        public void HangUp()
        {
           CallNumber = string.Empty;

           if (phoneState == PhoneState.InCall)
           {
               sipClient.HangUp();
               phoneState = PhoneState.Normal;
           }
           else if(phoneState == PhoneState.InCall)
           {
               sipClient.Reject();
               phoneState = PhoneState.Normal;
           }
        }
        /// <summary>
        /// Starts to send a dtmf sign.
        /// </summary>
        /// <param name="btnNum">The sended Dtmf sign.</param>
        public void BtnMouseLeftUp(int btnNum)
        {
            if (phoneState == PhoneState.InCall)
                sipClient.StopDtmf(btnNum);
        }

        /// <summary>
        /// Atops to send  the dtmf sign.
        /// </summary>
        /// <param name="btnNum">The sended Dtmf sign.</param>
        public void BtnMouseLeftDown(int btnNum)
        {
            if (phoneState == PhoneState.InCall)
                sipClient.StartDtmf(btnNum);
        }


        #endregion

        #region Properties

        /// <summary>
        /// Registered SIP username
        /// </summary>
        private string _lblUserName;
        public string UserName
        {
            get { return _lblUserName; }
            set { _lblUserName = value; OnPropertyChanged("UserName"); }
        }
        /// <summary>
        /// Connection status with server
        /// </summary>
        private string _lblConnectionStatus;
        public string ConnectionStatus
        {
            get { return _lblConnectionStatus; }
            set { _lblConnectionStatus = value; OnPropertyChanged("ConnectionStatus"); }
        }
        /// <summary>
        /// Current call info
        /// </summary>
        private string _lblCallInfo;
        public string CallInfo
        {
            get { return _lblCallInfo; }
            set { _lblCallInfo = value; OnPropertyChanged("CallInfo"); }
        }
        /// <summary>
        /// The dial number
        /// </summary>
        private string _callNumber;
        public string CallNumber
        {
            get
            {
                return _callNumber;
            }
            set { _callNumber = value; OnPropertyChanged("CallNumber"); }
        }

        #endregion

        #region Implementation of INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        protected virtual void OnPropertyChanged(string property)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(property));
        }

        private void hyperBtn_Click(object sender, RoutedEventArgs e)
        {
            HtmlPage.Window.Navigate(new Uri("http://www.voip-sip-sdk.com"), "_blank");
        }       
    }
}
